# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Chat Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.models_dictionary_child import ModelsDictionaryChild


class ModelsDictionaryWithChildren(Model):
    """Models dictionary with children (models.DictionaryWithChildren)

    Properties:
        id_: (id) REQUIRED str

        namespace: (namespace) REQUIRED str

        parent_id: (parentId) REQUIRED str

        word: (word) REQUIRED str

        word_type: (wordType) REQUIRED str

        false_negatives: (falseNegatives) OPTIONAL List[ModelsDictionaryChild]

        false_positives: (falsePositives) OPTIONAL List[ModelsDictionaryChild]
    """

    # region fields

    id_: str  # REQUIRED
    namespace: str  # REQUIRED
    parent_id: str  # REQUIRED
    word: str  # REQUIRED
    word_type: str  # REQUIRED
    false_negatives: List[ModelsDictionaryChild]  # OPTIONAL
    false_positives: List[ModelsDictionaryChild]  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_id(self, value: str) -> ModelsDictionaryWithChildren:
        self.id_ = value
        return self

    def with_namespace(self, value: str) -> ModelsDictionaryWithChildren:
        self.namespace = value
        return self

    def with_parent_id(self, value: str) -> ModelsDictionaryWithChildren:
        self.parent_id = value
        return self

    def with_word(self, value: str) -> ModelsDictionaryWithChildren:
        self.word = value
        return self

    def with_word_type(self, value: str) -> ModelsDictionaryWithChildren:
        self.word_type = value
        return self

    def with_false_negatives(
        self, value: List[ModelsDictionaryChild]
    ) -> ModelsDictionaryWithChildren:
        self.false_negatives = value
        return self

    def with_false_positives(
        self, value: List[ModelsDictionaryChild]
    ) -> ModelsDictionaryWithChildren:
        self.false_positives = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "id_"):
            result["id"] = str(self.id_)
        elif include_empty:
            result["id"] = ""
        if hasattr(self, "namespace"):
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "parent_id"):
            result["parentId"] = str(self.parent_id)
        elif include_empty:
            result["parentId"] = ""
        if hasattr(self, "word"):
            result["word"] = str(self.word)
        elif include_empty:
            result["word"] = ""
        if hasattr(self, "word_type"):
            result["wordType"] = str(self.word_type)
        elif include_empty:
            result["wordType"] = ""
        if hasattr(self, "false_negatives"):
            result["falseNegatives"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.false_negatives
            ]
        elif include_empty:
            result["falseNegatives"] = []
        if hasattr(self, "false_positives"):
            result["falsePositives"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.false_positives
            ]
        elif include_empty:
            result["falsePositives"] = []
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        id_: str,
        namespace: str,
        parent_id: str,
        word: str,
        word_type: str,
        false_negatives: Optional[List[ModelsDictionaryChild]] = None,
        false_positives: Optional[List[ModelsDictionaryChild]] = None,
        **kwargs,
    ) -> ModelsDictionaryWithChildren:
        instance = cls()
        instance.id_ = id_
        instance.namespace = namespace
        instance.parent_id = parent_id
        instance.word = word
        instance.word_type = word_type
        if false_negatives is not None:
            instance.false_negatives = false_negatives
        if false_positives is not None:
            instance.false_positives = false_positives
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelsDictionaryWithChildren:
        instance = cls()
        if not dict_:
            return instance
        if "id" in dict_ and dict_["id"] is not None:
            instance.id_ = str(dict_["id"])
        elif include_empty:
            instance.id_ = ""
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "parentId" in dict_ and dict_["parentId"] is not None:
            instance.parent_id = str(dict_["parentId"])
        elif include_empty:
            instance.parent_id = ""
        if "word" in dict_ and dict_["word"] is not None:
            instance.word = str(dict_["word"])
        elif include_empty:
            instance.word = ""
        if "wordType" in dict_ and dict_["wordType"] is not None:
            instance.word_type = str(dict_["wordType"])
        elif include_empty:
            instance.word_type = ""
        if "falseNegatives" in dict_ and dict_["falseNegatives"] is not None:
            instance.false_negatives = [
                ModelsDictionaryChild.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["falseNegatives"]
            ]
        elif include_empty:
            instance.false_negatives = []
        if "falsePositives" in dict_ and dict_["falsePositives"] is not None:
            instance.false_positives = [
                ModelsDictionaryChild.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["falsePositives"]
            ]
        elif include_empty:
            instance.false_positives = []
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelsDictionaryWithChildren]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelsDictionaryWithChildren]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ModelsDictionaryWithChildren,
        List[ModelsDictionaryWithChildren],
        Dict[Any, ModelsDictionaryWithChildren],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "id": "id_",
            "namespace": "namespace",
            "parentId": "parent_id",
            "word": "word",
            "wordType": "word_type",
            "falseNegatives": "false_negatives",
            "falsePositives": "false_positives",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "id": True,
            "namespace": True,
            "parentId": True,
            "word": True,
            "wordType": True,
            "falseNegatives": False,
            "falsePositives": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Platform Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model
from ....core import StrEnum


class DebitBalanceSourceEnum(StrEnum):
    DLC_REVOCATION = "DLC_REVOCATION"
    EXPIRATION = "EXPIRATION"
    IAP_REVOCATION = "IAP_REVOCATION"
    ORDER_REVOCATION = "ORDER_REVOCATION"
    OTHER = "OTHER"
    PAYMENT = "PAYMENT"
    TRADE = "TRADE"


class WalletPlatformEnum(StrEnum):
    EPIC = "Epic"
    GOOGLEPLAY = "GooglePlay"
    IOS = "IOS"
    NINTENDO = "Nintendo"
    OCULUS = "Oculus"
    OTHER = "Other"
    PLAYSTATION = "Playstation"
    STEAM = "Steam"
    XBOX = "Xbox"


class DebitByWalletPlatformRequest(Model):
    """Debit by wallet platform request (DebitByWalletPlatformRequest)

    Properties:
        amount: (amount) REQUIRED int

        debit_balance_source: (debitBalanceSource) OPTIONAL Union[str, DebitBalanceSourceEnum]

        metadata: (metadata) OPTIONAL Dict[str, Any]

        reason: (reason) OPTIONAL str

        wallet_platform: (walletPlatform) OPTIONAL Union[str, WalletPlatformEnum]
    """

    # region fields

    amount: int  # REQUIRED
    debit_balance_source: Union[str, DebitBalanceSourceEnum]  # OPTIONAL
    metadata: Dict[str, Any]  # OPTIONAL
    reason: str  # OPTIONAL
    wallet_platform: Union[str, WalletPlatformEnum]  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_amount(self, value: int) -> DebitByWalletPlatformRequest:
        self.amount = value
        return self

    def with_debit_balance_source(
        self, value: Union[str, DebitBalanceSourceEnum]
    ) -> DebitByWalletPlatformRequest:
        self.debit_balance_source = value
        return self

    def with_metadata(self, value: Dict[str, Any]) -> DebitByWalletPlatformRequest:
        self.metadata = value
        return self

    def with_reason(self, value: str) -> DebitByWalletPlatformRequest:
        self.reason = value
        return self

    def with_wallet_platform(
        self, value: Union[str, WalletPlatformEnum]
    ) -> DebitByWalletPlatformRequest:
        self.wallet_platform = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "amount"):
            result["amount"] = int(self.amount)
        elif include_empty:
            result["amount"] = 0
        if hasattr(self, "debit_balance_source"):
            result["debitBalanceSource"] = str(self.debit_balance_source)
        elif include_empty:
            result["debitBalanceSource"] = Union[str, DebitBalanceSourceEnum]()
        if hasattr(self, "metadata"):
            result["metadata"] = {str(k0): v0 for k0, v0 in self.metadata.items()}
        elif include_empty:
            result["metadata"] = {}
        if hasattr(self, "reason"):
            result["reason"] = str(self.reason)
        elif include_empty:
            result["reason"] = ""
        if hasattr(self, "wallet_platform"):
            result["walletPlatform"] = str(self.wallet_platform)
        elif include_empty:
            result["walletPlatform"] = Union[str, WalletPlatformEnum]()
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        amount: int,
        debit_balance_source: Optional[Union[str, DebitBalanceSourceEnum]] = None,
        metadata: Optional[Dict[str, Any]] = None,
        reason: Optional[str] = None,
        wallet_platform: Optional[Union[str, WalletPlatformEnum]] = None,
        **kwargs,
    ) -> DebitByWalletPlatformRequest:
        instance = cls()
        instance.amount = amount
        if debit_balance_source is not None:
            instance.debit_balance_source = debit_balance_source
        if metadata is not None:
            instance.metadata = metadata
        if reason is not None:
            instance.reason = reason
        if wallet_platform is not None:
            instance.wallet_platform = wallet_platform
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> DebitByWalletPlatformRequest:
        instance = cls()
        if not dict_:
            return instance
        if "amount" in dict_ and dict_["amount"] is not None:
            instance.amount = int(dict_["amount"])
        elif include_empty:
            instance.amount = 0
        if "debitBalanceSource" in dict_ and dict_["debitBalanceSource"] is not None:
            instance.debit_balance_source = str(dict_["debitBalanceSource"])
        elif include_empty:
            instance.debit_balance_source = Union[str, DebitBalanceSourceEnum]()
        if "metadata" in dict_ and dict_["metadata"] is not None:
            instance.metadata = {str(k0): v0 for k0, v0 in dict_["metadata"].items()}
        elif include_empty:
            instance.metadata = {}
        if "reason" in dict_ and dict_["reason"] is not None:
            instance.reason = str(dict_["reason"])
        elif include_empty:
            instance.reason = ""
        if "walletPlatform" in dict_ and dict_["walletPlatform"] is not None:
            instance.wallet_platform = str(dict_["walletPlatform"])
        elif include_empty:
            instance.wallet_platform = Union[str, WalletPlatformEnum]()
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, DebitByWalletPlatformRequest]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[DebitByWalletPlatformRequest]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        DebitByWalletPlatformRequest,
        List[DebitByWalletPlatformRequest],
        Dict[Any, DebitByWalletPlatformRequest],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "amount": "amount",
            "debitBalanceSource": "debit_balance_source",
            "metadata": "metadata",
            "reason": "reason",
            "walletPlatform": "wallet_platform",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "amount": True,
            "debitBalanceSource": False,
            "metadata": False,
            "reason": False,
            "walletPlatform": False,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "debitBalanceSource": [
                "DLC_REVOCATION",
                "EXPIRATION",
                "IAP_REVOCATION",
                "ORDER_REVOCATION",
                "OTHER",
                "PAYMENT",
                "TRADE",
            ],
            "walletPlatform": [
                "Epic",
                "GooglePlay",
                "IOS",
                "Nintendo",
                "Oculus",
                "Other",
                "Playstation",
                "Steam",
                "Xbox",
            ],
        }

    # endregion static methods



from datetime import datetime
from typing import List, Optional

from ._ctypes import PermissionStruct


def replace_resource(
    resource: str,
    namespace: Optional[str] = None,
    token_namespace: Optional[str] = None,
    publisher_namespace: Optional[str] = None,
    user_id: Optional[str] = None,
    cross_allowed: bool = False,
) -> str:
    new_resource = resource

    if (
        cross_allowed
        and token_namespace is not None
        and (publisher_namespace == token_namespace or publisher_namespace == namespace)
    ):
        new_resource = new_resource.replace("{namespace}", token_namespace)

    if namespace is not None:
        new_resource = new_resource.replace("{namespace}", namespace)

    if user_id is not None:
        new_resource = new_resource.replace("{userId}", user_id)

    return new_resource


def str2datetime(s: str) -> datetime:
    # from: 'YYYY-mm-ddTHH:MM:SS.fffffffffZ'
    # to:   'YYYY-mm-ddTHH:MM:SS.fffZ+0000'
    tz = "Z+0000" if s.endswith("Z") else ""  # Add explicit UTC timezone.
    s = s[0:23] + tz
    return datetime.strptime(s, "%Y-%m-%dT%H:%M:%S.%fZ%z")


def validate_resource(resource: str, expected_resource: str) -> bool:
    # Split both the resource and the expected resource into tokens with ':' as delimiter.
    r_items = resource.split(":")
    e_items = expected_resource.split(":")

    r_items_len = len(r_items)
    e_items_len = len(e_items)
    min_len = min(r_items_len, e_items_len)

    # Compare each token in the resource and the expected resource from left to right.
    for i in range(min_len):
        r_token = r_items[i]
        e_token = e_items[i]
        # If token string match or other one is '*', then proceed to next token.
        # Otherwise, we can stop here because the resource does not match.
        # |                       vvv : DOES NOT MATCH
        # | resource(1):          BAR
        # | expected_resource(1): FOO
        if r_token != e_token and r_token != "*":
            return False

    # If the number of resource tokens < the number of expected resource tokens.
    if r_items_len < e_items_len:
        # If the last resource token is '*',
        # we can stop here because the resource does not match.
        # |                           vvv : DOES NOT MATCH!
        # | resource(2):          FOO:BAR
        # | expected_resource(3): FOO:BAR:BAZ
        if r_items[-1] != "*":
            return False

        # If the second resource token from the right is 'NAMESPACE' or 'USER',
        # we can stop here because the resource does not match.
        # |                           vvvv : DOES NOT MATCH
        # | resource(3):          FOO:USER:*
        # | expected_resource(4): FOO:BAR:BAZ:QUX
        elif r_items_len >= 2 and r_items[-2] in ("NAMESPACE", "USER"):
            return False

    # If the number resource tokens > the number of expected resource tokens.
    if r_items_len > e_items_len:
        for i in range(e_items_len, r_items_len):
            # If the remaining resource tokens are ALL '*', then proceed.
            # Otherwise, we can stop here and the resource does not match.
            # |                           vvv : DOES NOT MATCH!
            # | resource(2):          FOO:BAR
            # | expected_resource(1): FOO
            if r_items[i] != "*":
                return False

    return True


def validate_action(action: int, expected_action: int) -> bool:
    return (action & expected_action) == expected_action


def validate_permission(
    target: PermissionStruct, permissions: List[PermissionStruct]
) -> bool:
    for permission in permissions:
        if validate_action(permission.action, target.action) and validate_resource(
            permission.resource, target.resource
        ):
            return True

    return False



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

# Fleet Commander

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from .....core import Operation
from .....core import HeaderStr
from .....core import HttpResponse

from ...models import ApiArtifactResponse
from ...models import ResponseErrorResponse


class ArtifactGet(Operation):
    """get all artifacts matching the provided criteria (ArtifactGet)

    Get all artifacts matching the provided search criteria. When criteria is not specified the data returned won't have been filtered on those parameters

    Required Permission: ADMIN:NAMESPACE:{namespace}:AMS:ARTIFACT [READ]

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:AMS:ARTIFACT [READ]

    Properties:
        url: /ams/v1/admin/namespaces/{namespace}/artifacts

        method: GET

        tags: ["Artifacts"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        artifact_type: (artifactType) OPTIONAL str in query

        count: (count) OPTIONAL int in query

        end_date: (endDate) OPTIONAL str in query

        fleet_id: (fleetID) OPTIONAL str in query

        image_id: (imageID) OPTIONAL str in query

        max_size: (maxSize) OPTIONAL int in query

        min_size: (minSize) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        region: (region) OPTIONAL str in query

        server_id: (serverId) OPTIONAL str in query

        start_date: (startDate) OPTIONAL str in query

        status: (status) OPTIONAL str in query

    Responses:
        200: OK - List[ApiArtifactResponse] (success)

        400: Bad Request - ResponseErrorResponse (invalid data in request)

        401: Unauthorized - ResponseErrorResponse (no authorization provided)

        403: Forbidden - ResponseErrorResponse (insufficient permissions)

        500: Internal Server Error - ResponseErrorResponse (internal server error)
    """

    # region fields

    _url: str = "/ams/v1/admin/namespaces/{namespace}/artifacts"
    _method: str = "GET"
    _consumes: List[str] = ["application/json"]
    _produces: List[str] = ["application/json"]
    _securities: List[List[str]] = [["BEARER_AUTH"]]
    _location_query: str = None

    namespace: str  # REQUIRED in [path]
    artifact_type: str  # OPTIONAL in [query]
    count: int  # OPTIONAL in [query]
    end_date: str  # OPTIONAL in [query]
    fleet_id: str  # OPTIONAL in [query]
    image_id: str  # OPTIONAL in [query]
    max_size: int  # OPTIONAL in [query]
    min_size: int  # OPTIONAL in [query]
    offset: int  # OPTIONAL in [query]
    region: str  # OPTIONAL in [query]
    server_id: str  # OPTIONAL in [query]
    start_date: str  # OPTIONAL in [query]
    status: str  # OPTIONAL in [query]

    # endregion fields

    # region properties

    @property
    def url(self) -> str:
        return self._url

    @property
    def method(self) -> str:
        return self._method

    @property
    def consumes(self) -> List[str]:
        return self._consumes

    @property
    def produces(self) -> List[str]:
        return self._produces

    @property
    def securities(self) -> List[List[str]]:
        return self._securities

    @property
    def location_query(self) -> str:
        return self._location_query

    # endregion properties

    # region get methods

    # endregion get methods

    # region get_x_params methods

    def get_all_params(self) -> dict:
        return {
            "path": self.get_path_params(),
            "query": self.get_query_params(),
        }

    def get_path_params(self) -> dict:
        result = {}
        if hasattr(self, "namespace"):
            result["namespace"] = self.namespace
        return result

    def get_query_params(self) -> dict:
        result = {}
        if hasattr(self, "artifact_type"):
            result["artifactType"] = self.artifact_type
        if hasattr(self, "count"):
            result["count"] = self.count
        if hasattr(self, "end_date"):
            result["endDate"] = self.end_date
        if hasattr(self, "fleet_id"):
            result["fleetID"] = self.fleet_id
        if hasattr(self, "image_id"):
            result["imageID"] = self.image_id
        if hasattr(self, "max_size"):
            result["maxSize"] = self.max_size
        if hasattr(self, "min_size"):
            result["minSize"] = self.min_size
        if hasattr(self, "offset"):
            result["offset"] = self.offset
        if hasattr(self, "region"):
            result["region"] = self.region
        if hasattr(self, "server_id"):
            result["serverId"] = self.server_id
        if hasattr(self, "start_date"):
            result["startDate"] = self.start_date
        if hasattr(self, "status"):
            result["status"] = self.status
        return result

    # endregion get_x_params methods

    # region is/has methods

    # endregion is/has methods

    # region with_x methods

    def with_namespace(self, value: str) -> ArtifactGet:
        self.namespace = value
        return self

    def with_artifact_type(self, value: str) -> ArtifactGet:
        self.artifact_type = value
        return self

    def with_count(self, value: int) -> ArtifactGet:
        self.count = value
        return self

    def with_end_date(self, value: str) -> ArtifactGet:
        self.end_date = value
        return self

    def with_fleet_id(self, value: str) -> ArtifactGet:
        self.fleet_id = value
        return self

    def with_image_id(self, value: str) -> ArtifactGet:
        self.image_id = value
        return self

    def with_max_size(self, value: int) -> ArtifactGet:
        self.max_size = value
        return self

    def with_min_size(self, value: int) -> ArtifactGet:
        self.min_size = value
        return self

    def with_offset(self, value: int) -> ArtifactGet:
        self.offset = value
        return self

    def with_region(self, value: str) -> ArtifactGet:
        self.region = value
        return self

    def with_server_id(self, value: str) -> ArtifactGet:
        self.server_id = value
        return self

    def with_start_date(self, value: str) -> ArtifactGet:
        self.start_date = value
        return self

    def with_status(self, value: str) -> ArtifactGet:
        self.status = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "namespace") and self.namespace:
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "artifact_type") and self.artifact_type:
            result["artifactType"] = str(self.artifact_type)
        elif include_empty:
            result["artifactType"] = ""
        if hasattr(self, "count") and self.count:
            result["count"] = int(self.count)
        elif include_empty:
            result["count"] = 0
        if hasattr(self, "end_date") and self.end_date:
            result["endDate"] = str(self.end_date)
        elif include_empty:
            result["endDate"] = ""
        if hasattr(self, "fleet_id") and self.fleet_id:
            result["fleetID"] = str(self.fleet_id)
        elif include_empty:
            result["fleetID"] = ""
        if hasattr(self, "image_id") and self.image_id:
            result["imageID"] = str(self.image_id)
        elif include_empty:
            result["imageID"] = ""
        if hasattr(self, "max_size") and self.max_size:
            result["maxSize"] = int(self.max_size)
        elif include_empty:
            result["maxSize"] = 0
        if hasattr(self, "min_size") and self.min_size:
            result["minSize"] = int(self.min_size)
        elif include_empty:
            result["minSize"] = 0
        if hasattr(self, "offset") and self.offset:
            result["offset"] = int(self.offset)
        elif include_empty:
            result["offset"] = 0
        if hasattr(self, "region") and self.region:
            result["region"] = str(self.region)
        elif include_empty:
            result["region"] = ""
        if hasattr(self, "server_id") and self.server_id:
            result["serverId"] = str(self.server_id)
        elif include_empty:
            result["serverId"] = ""
        if hasattr(self, "start_date") and self.start_date:
            result["startDate"] = str(self.start_date)
        elif include_empty:
            result["startDate"] = ""
        if hasattr(self, "status") and self.status:
            result["status"] = str(self.status)
        elif include_empty:
            result["status"] = ""
        return result

    # endregion to methods

    # region response methods

    # noinspection PyMethodMayBeStatic
    def parse_response(
        self, code: int, content_type: str, content: Any
    ) -> Tuple[
        Union[None, List[ApiArtifactResponse]],
        Union[None, HttpResponse, ResponseErrorResponse],
    ]:
        """Parse the given response.

        200: OK - List[ApiArtifactResponse] (success)

        400: Bad Request - ResponseErrorResponse (invalid data in request)

        401: Unauthorized - ResponseErrorResponse (no authorization provided)

        403: Forbidden - ResponseErrorResponse (insufficient permissions)

        500: Internal Server Error - ResponseErrorResponse (internal server error)

        ---: HttpResponse (Undocumented Response)

        ---: HttpResponse (Unexpected Content-Type Error)

        ---: HttpResponse (Unhandled Error)
        """
        pre_processed_response, error = self.pre_process_response(
            code=code, content_type=content_type, content=content
        )
        if error is not None:
            return None, None if error.is_no_content() else error
        code, content_type, content = pre_processed_response

        if code == 200:
            return [ApiArtifactResponse.create_from_dict(i) for i in content], None
        if code == 400:
            return None, ResponseErrorResponse.create_from_dict(content)
        if code == 401:
            return None, ResponseErrorResponse.create_from_dict(content)
        if code == 403:
            return None, ResponseErrorResponse.create_from_dict(content)
        if code == 500:
            return None, ResponseErrorResponse.create_from_dict(content)

        return self.handle_undocumented_response(
            code=code, content_type=content_type, content=content
        )

    # endregion response methods

    # region static methods

    @classmethod
    def create(
        cls,
        namespace: str,
        artifact_type: Optional[str] = None,
        count: Optional[int] = None,
        end_date: Optional[str] = None,
        fleet_id: Optional[str] = None,
        image_id: Optional[str] = None,
        max_size: Optional[int] = None,
        min_size: Optional[int] = None,
        offset: Optional[int] = None,
        region: Optional[str] = None,
        server_id: Optional[str] = None,
        start_date: Optional[str] = None,
        status: Optional[str] = None,
        **kwargs,
    ) -> ArtifactGet:
        instance = cls()
        instance.namespace = namespace
        if artifact_type is not None:
            instance.artifact_type = artifact_type
        if count is not None:
            instance.count = count
        if end_date is not None:
            instance.end_date = end_date
        if fleet_id is not None:
            instance.fleet_id = fleet_id
        if image_id is not None:
            instance.image_id = image_id
        if max_size is not None:
            instance.max_size = max_size
        if min_size is not None:
            instance.min_size = min_size
        if offset is not None:
            instance.offset = offset
        if region is not None:
            instance.region = region
        if server_id is not None:
            instance.server_id = server_id
        if start_date is not None:
            instance.start_date = start_date
        if status is not None:
            instance.status = status
        if x_flight_id := kwargs.get("x_flight_id", None):
            instance.x_flight_id = x_flight_id
        return instance

    @classmethod
    def create_from_dict(cls, dict_: dict, include_empty: bool = False) -> ArtifactGet:
        instance = cls()
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "artifactType" in dict_ and dict_["artifactType"] is not None:
            instance.artifact_type = str(dict_["artifactType"])
        elif include_empty:
            instance.artifact_type = ""
        if "count" in dict_ and dict_["count"] is not None:
            instance.count = int(dict_["count"])
        elif include_empty:
            instance.count = 0
        if "endDate" in dict_ and dict_["endDate"] is not None:
            instance.end_date = str(dict_["endDate"])
        elif include_empty:
            instance.end_date = ""
        if "fleetID" in dict_ and dict_["fleetID"] is not None:
            instance.fleet_id = str(dict_["fleetID"])
        elif include_empty:
            instance.fleet_id = ""
        if "imageID" in dict_ and dict_["imageID"] is not None:
            instance.image_id = str(dict_["imageID"])
        elif include_empty:
            instance.image_id = ""
        if "maxSize" in dict_ and dict_["maxSize"] is not None:
            instance.max_size = int(dict_["maxSize"])
        elif include_empty:
            instance.max_size = 0
        if "minSize" in dict_ and dict_["minSize"] is not None:
            instance.min_size = int(dict_["minSize"])
        elif include_empty:
            instance.min_size = 0
        if "offset" in dict_ and dict_["offset"] is not None:
            instance.offset = int(dict_["offset"])
        elif include_empty:
            instance.offset = 0
        if "region" in dict_ and dict_["region"] is not None:
            instance.region = str(dict_["region"])
        elif include_empty:
            instance.region = ""
        if "serverId" in dict_ and dict_["serverId"] is not None:
            instance.server_id = str(dict_["serverId"])
        elif include_empty:
            instance.server_id = ""
        if "startDate" in dict_ and dict_["startDate"] is not None:
            instance.start_date = str(dict_["startDate"])
        elif include_empty:
            instance.start_date = ""
        if "status" in dict_ and dict_["status"] is not None:
            instance.status = str(dict_["status"])
        elif include_empty:
            instance.status = ""
        return instance

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "namespace": "namespace",
            "artifactType": "artifact_type",
            "count": "count",
            "endDate": "end_date",
            "fleetID": "fleet_id",
            "imageID": "image_id",
            "maxSize": "max_size",
            "minSize": "min_size",
            "offset": "offset",
            "region": "region",
            "serverId": "server_id",
            "startDate": "start_date",
            "status": "status",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "namespace": True,
            "artifactType": False,
            "count": False,
            "endDate": False,
            "fleetID": False,
            "imageID": False,
            "maxSize": False,
            "minSize": False,
            "offset": False,
            "region": False,
            "serverId": False,
            "startDate": False,
            "status": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Event Log Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class ModelsEventRegistry(Model):
    """Models event registry (models.EventRegistry)

    Properties:
        event_id: (EventID) REQUIRED int

        event_level: (EventLevel) REQUIRED int

        event_type: (EventType) REQUIRED int

        ux: (UX) REQUIRED int
    """

    # region fields

    event_id: int  # REQUIRED
    event_level: int  # REQUIRED
    event_type: int  # REQUIRED
    ux: int  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_event_id(self, value: int) -> ModelsEventRegistry:
        self.event_id = value
        return self

    def with_event_level(self, value: int) -> ModelsEventRegistry:
        self.event_level = value
        return self

    def with_event_type(self, value: int) -> ModelsEventRegistry:
        self.event_type = value
        return self

    def with_ux(self, value: int) -> ModelsEventRegistry:
        self.ux = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "event_id"):
            result["EventID"] = int(self.event_id)
        elif include_empty:
            result["EventID"] = 0
        if hasattr(self, "event_level"):
            result["EventLevel"] = int(self.event_level)
        elif include_empty:
            result["EventLevel"] = 0
        if hasattr(self, "event_type"):
            result["EventType"] = int(self.event_type)
        elif include_empty:
            result["EventType"] = 0
        if hasattr(self, "ux"):
            result["UX"] = int(self.ux)
        elif include_empty:
            result["UX"] = 0
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls, event_id: int, event_level: int, event_type: int, ux: int, **kwargs
    ) -> ModelsEventRegistry:
        instance = cls()
        instance.event_id = event_id
        instance.event_level = event_level
        instance.event_type = event_type
        instance.ux = ux
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelsEventRegistry:
        instance = cls()
        if not dict_:
            return instance
        if "EventID" in dict_ and dict_["EventID"] is not None:
            instance.event_id = int(dict_["EventID"])
        elif include_empty:
            instance.event_id = 0
        if "EventLevel" in dict_ and dict_["EventLevel"] is not None:
            instance.event_level = int(dict_["EventLevel"])
        elif include_empty:
            instance.event_level = 0
        if "EventType" in dict_ and dict_["EventType"] is not None:
            instance.event_type = int(dict_["EventType"])
        elif include_empty:
            instance.event_type = 0
        if "UX" in dict_ and dict_["UX"] is not None:
            instance.ux = int(dict_["UX"])
        elif include_empty:
            instance.ux = 0
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelsEventRegistry]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelsEventRegistry]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ModelsEventRegistry, List[ModelsEventRegistry], Dict[Any, ModelsEventRegistry]
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "EventID": "event_id",
            "EventLevel": "event_level",
            "EventType": "event_type",
            "UX": "ux",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "EventID": True,
            "EventLevel": True,
            "EventType": True,
            "UX": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Iam Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.accountcommon_user_linked_platform import AccountcommonUserLinkedPlatform


class AccountcommonUserSearchByPlatformIDResult(Model):
    """Accountcommon user search by platform ID result (accountcommon.UserSearchByPlatformIDResult)

    Properties:
        display_name: (DisplayName) REQUIRED str

        email_address: (EmailAddress) REQUIRED str

        linked_platforms: (LinkedPlatforms) REQUIRED List[AccountcommonUserLinkedPlatform]

        phone_number: (PhoneNumber) REQUIRED str

        user_id: (UserId) REQUIRED str
    """

    # region fields

    display_name: str  # REQUIRED
    email_address: str  # REQUIRED
    linked_platforms: List[AccountcommonUserLinkedPlatform]  # REQUIRED
    phone_number: str  # REQUIRED
    user_id: str  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_display_name(
        self, value: str
    ) -> AccountcommonUserSearchByPlatformIDResult:
        self.display_name = value
        return self

    def with_email_address(
        self, value: str
    ) -> AccountcommonUserSearchByPlatformIDResult:
        self.email_address = value
        return self

    def with_linked_platforms(
        self, value: List[AccountcommonUserLinkedPlatform]
    ) -> AccountcommonUserSearchByPlatformIDResult:
        self.linked_platforms = value
        return self

    def with_phone_number(
        self, value: str
    ) -> AccountcommonUserSearchByPlatformIDResult:
        self.phone_number = value
        return self

    def with_user_id(self, value: str) -> AccountcommonUserSearchByPlatformIDResult:
        self.user_id = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "display_name"):
            result["DisplayName"] = str(self.display_name)
        elif include_empty:
            result["DisplayName"] = ""
        if hasattr(self, "email_address"):
            result["EmailAddress"] = str(self.email_address)
        elif include_empty:
            result["EmailAddress"] = ""
        if hasattr(self, "linked_platforms"):
            result["LinkedPlatforms"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.linked_platforms
            ]
        elif include_empty:
            result["LinkedPlatforms"] = []
        if hasattr(self, "phone_number"):
            result["PhoneNumber"] = str(self.phone_number)
        elif include_empty:
            result["PhoneNumber"] = ""
        if hasattr(self, "user_id"):
            result["UserId"] = str(self.user_id)
        elif include_empty:
            result["UserId"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        display_name: str,
        email_address: str,
        linked_platforms: List[AccountcommonUserLinkedPlatform],
        phone_number: str,
        user_id: str,
        **kwargs,
    ) -> AccountcommonUserSearchByPlatformIDResult:
        instance = cls()
        instance.display_name = display_name
        instance.email_address = email_address
        instance.linked_platforms = linked_platforms
        instance.phone_number = phone_number
        instance.user_id = user_id
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> AccountcommonUserSearchByPlatformIDResult:
        instance = cls()
        if not dict_:
            return instance
        if "DisplayName" in dict_ and dict_["DisplayName"] is not None:
            instance.display_name = str(dict_["DisplayName"])
        elif include_empty:
            instance.display_name = ""
        if "EmailAddress" in dict_ and dict_["EmailAddress"] is not None:
            instance.email_address = str(dict_["EmailAddress"])
        elif include_empty:
            instance.email_address = ""
        if "LinkedPlatforms" in dict_ and dict_["LinkedPlatforms"] is not None:
            instance.linked_platforms = [
                AccountcommonUserLinkedPlatform.create_from_dict(
                    i0, include_empty=include_empty
                )
                for i0 in dict_["LinkedPlatforms"]
            ]
        elif include_empty:
            instance.linked_platforms = []
        if "PhoneNumber" in dict_ and dict_["PhoneNumber"] is not None:
            instance.phone_number = str(dict_["PhoneNumber"])
        elif include_empty:
            instance.phone_number = ""
        if "UserId" in dict_ and dict_["UserId"] is not None:
            instance.user_id = str(dict_["UserId"])
        elif include_empty:
            instance.user_id = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, AccountcommonUserSearchByPlatformIDResult]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[AccountcommonUserSearchByPlatformIDResult]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        AccountcommonUserSearchByPlatformIDResult,
        List[AccountcommonUserSearchByPlatformIDResult],
        Dict[Any, AccountcommonUserSearchByPlatformIDResult],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "DisplayName": "display_name",
            "EmailAddress": "email_address",
            "LinkedPlatforms": "linked_platforms",
            "PhoneNumber": "phone_number",
            "UserId": "user_id",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "DisplayName": True,
            "EmailAddress": True,
            "LinkedPlatforms": True,
            "PhoneNumber": True,
            "UserId": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Platform Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class KeyGroupDynamicInfo(Model):
    """Key group dynamic info (KeyGroupDynamicInfo)

    Properties:
        available_sale_count: (availableSaleCount) REQUIRED int

        quantity: (quantity) REQUIRED int

        sale_count: (saleCount) REQUIRED int
    """

    # region fields

    available_sale_count: int  # REQUIRED
    quantity: int  # REQUIRED
    sale_count: int  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_available_sale_count(self, value: int) -> KeyGroupDynamicInfo:
        self.available_sale_count = value
        return self

    def with_quantity(self, value: int) -> KeyGroupDynamicInfo:
        self.quantity = value
        return self

    def with_sale_count(self, value: int) -> KeyGroupDynamicInfo:
        self.sale_count = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "available_sale_count"):
            result["availableSaleCount"] = int(self.available_sale_count)
        elif include_empty:
            result["availableSaleCount"] = 0
        if hasattr(self, "quantity"):
            result["quantity"] = int(self.quantity)
        elif include_empty:
            result["quantity"] = 0
        if hasattr(self, "sale_count"):
            result["saleCount"] = int(self.sale_count)
        elif include_empty:
            result["saleCount"] = 0
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls, available_sale_count: int, quantity: int, sale_count: int, **kwargs
    ) -> KeyGroupDynamicInfo:
        instance = cls()
        instance.available_sale_count = available_sale_count
        instance.quantity = quantity
        instance.sale_count = sale_count
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> KeyGroupDynamicInfo:
        instance = cls()
        if not dict_:
            return instance
        if "availableSaleCount" in dict_ and dict_["availableSaleCount"] is not None:
            instance.available_sale_count = int(dict_["availableSaleCount"])
        elif include_empty:
            instance.available_sale_count = 0
        if "quantity" in dict_ and dict_["quantity"] is not None:
            instance.quantity = int(dict_["quantity"])
        elif include_empty:
            instance.quantity = 0
        if "saleCount" in dict_ and dict_["saleCount"] is not None:
            instance.sale_count = int(dict_["saleCount"])
        elif include_empty:
            instance.sale_count = 0
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, KeyGroupDynamicInfo]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[KeyGroupDynamicInfo]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        KeyGroupDynamicInfo, List[KeyGroupDynamicInfo], Dict[Any, KeyGroupDynamicInfo]
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "availableSaleCount": "available_sale_count",
            "quantity": "quantity",
            "saleCount": "sale_count",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "availableSaleCount": True,
            "quantity": True,
            "saleCount": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Matchmaking Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model
from ....core import StrEnum


class SelectionEnum(StrEnum):
    NEWEST = "newest"
    OLDEST = "oldest"
    PIVOT = "pivot"
    RANDOM = "random"


class ModelsSelectionRule(Model):
    """Models selection rule (models.SelectionRule)

    Properties:
        selection: (selection) REQUIRED Union[str, SelectionEnum]

        threshold: (threshold) REQUIRED int
    """

    # region fields

    selection: Union[str, SelectionEnum]  # REQUIRED
    threshold: int  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_selection(self, value: Union[str, SelectionEnum]) -> ModelsSelectionRule:
        self.selection = value
        return self

    def with_threshold(self, value: int) -> ModelsSelectionRule:
        self.threshold = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "selection"):
            result["selection"] = str(self.selection)
        elif include_empty:
            result["selection"] = Union[str, SelectionEnum]()
        if hasattr(self, "threshold"):
            result["threshold"] = int(self.threshold)
        elif include_empty:
            result["threshold"] = 0
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls, selection: Union[str, SelectionEnum], threshold: int, **kwargs
    ) -> ModelsSelectionRule:
        instance = cls()
        instance.selection = selection
        instance.threshold = threshold
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelsSelectionRule:
        instance = cls()
        if not dict_:
            return instance
        if "selection" in dict_ and dict_["selection"] is not None:
            instance.selection = str(dict_["selection"])
        elif include_empty:
            instance.selection = Union[str, SelectionEnum]()
        if "threshold" in dict_ and dict_["threshold"] is not None:
            instance.threshold = int(dict_["threshold"])
        elif include_empty:
            instance.threshold = 0
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelsSelectionRule]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelsSelectionRule]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ModelsSelectionRule, List[ModelsSelectionRule], Dict[Any, ModelsSelectionRule]
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "selection": "selection",
            "threshold": "threshold",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "selection": True,
            "threshold": True,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "selection": ["newest", "oldest", "pivot", "random"],
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

# AccelByte Gaming Services Legal Service

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from .....core import Operation
from .....core import HeaderStr
from .....core import HttpResponse

from ...models import RetrieveLocalizedPolicyVersionResponse


class RetrieveLocalizedPolicyVersions1(Operation):
    """Retrieve Versions from Country-Specific Policy (retrieveLocalizedPolicyVersions_1)

    Retrieve versions of a particular country-specific policy.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:LEGAL", action=2 (READ)

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:LEGAL [READ]

    Properties:
        url: /agreement/admin/namespaces/{namespace}/localized-policy-versions/versions/{policyVersionId}

        method: GET

        tags: ["Localized Policy Versions With Namespace"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        policy_version_id: (policyVersionId) REQUIRED str in path

    Responses:
        200: OK - List[RetrieveLocalizedPolicyVersionResponse] (successful operation)
    """

    # region fields

    _url: str = "/agreement/admin/namespaces/{namespace}/localized-policy-versions/versions/{policyVersionId}"
    _method: str = "GET"
    _consumes: List[str] = []
    _produces: List[str] = ["application/json"]
    _securities: List[List[str]] = [["BEARER_AUTH"], ["BEARER_AUTH"]]
    _location_query: str = None

    namespace: str  # REQUIRED in [path]
    policy_version_id: str  # REQUIRED in [path]

    # endregion fields

    # region properties

    @property
    def url(self) -> str:
        return self._url

    @property
    def method(self) -> str:
        return self._method

    @property
    def consumes(self) -> List[str]:
        return self._consumes

    @property
    def produces(self) -> List[str]:
        return self._produces

    @property
    def securities(self) -> List[List[str]]:
        return self._securities

    @property
    def location_query(self) -> str:
        return self._location_query

    # endregion properties

    # region get methods

    # endregion get methods

    # region get_x_params methods

    def get_all_params(self) -> dict:
        return {
            "path": self.get_path_params(),
        }

    def get_path_params(self) -> dict:
        result = {}
        if hasattr(self, "namespace"):
            result["namespace"] = self.namespace
        if hasattr(self, "policy_version_id"):
            result["policyVersionId"] = self.policy_version_id
        return result

    # endregion get_x_params methods

    # region is/has methods

    # endregion is/has methods

    # region with_x methods

    def with_namespace(self, value: str) -> RetrieveLocalizedPolicyVersions1:
        self.namespace = value
        return self

    def with_policy_version_id(self, value: str) -> RetrieveLocalizedPolicyVersions1:
        self.policy_version_id = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "namespace") and self.namespace:
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "policy_version_id") and self.policy_version_id:
            result["policyVersionId"] = str(self.policy_version_id)
        elif include_empty:
            result["policyVersionId"] = ""
        return result

    # endregion to methods

    # region response methods

    # noinspection PyMethodMayBeStatic
    def parse_response(
        self, code: int, content_type: str, content: Any
    ) -> Tuple[
        Union[None, List[RetrieveLocalizedPolicyVersionResponse]],
        Union[None, HttpResponse],
    ]:
        """Parse the given response.

        200: OK - List[RetrieveLocalizedPolicyVersionResponse] (successful operation)

        ---: HttpResponse (Undocumented Response)

        ---: HttpResponse (Unexpected Content-Type Error)

        ---: HttpResponse (Unhandled Error)
        """
        pre_processed_response, error = self.pre_process_response(
            code=code, content_type=content_type, content=content
        )
        if error is not None:
            return None, None if error.is_no_content() else error
        code, content_type, content = pre_processed_response

        if code == 200:
            return [
                RetrieveLocalizedPolicyVersionResponse.create_from_dict(i)
                for i in content
            ], None

        return self.handle_undocumented_response(
            code=code, content_type=content_type, content=content
        )

    # endregion response methods

    # region static methods

    @classmethod
    def create(
        cls, namespace: str, policy_version_id: str, **kwargs
    ) -> RetrieveLocalizedPolicyVersions1:
        instance = cls()
        instance.namespace = namespace
        instance.policy_version_id = policy_version_id
        if x_flight_id := kwargs.get("x_flight_id", None):
            instance.x_flight_id = x_flight_id
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> RetrieveLocalizedPolicyVersions1:
        instance = cls()
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "policyVersionId" in dict_ and dict_["policyVersionId"] is not None:
            instance.policy_version_id = str(dict_["policyVersionId"])
        elif include_empty:
            instance.policy_version_id = ""
        return instance

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "namespace": "namespace",
            "policyVersionId": "policy_version_id",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "namespace": True,
            "policyVersionId": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Session Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.models_port_configuration_ams import ModelsPortConfigurationAMS


class ModelsGameServer(Model):
    """Models game server (models.GameServer)

    Properties:
        custom_attribute: (custom_attribute) REQUIRED str

        is_override_game_version: (is_override_game_version) REQUIRED bool

        last_update: (last_update) REQUIRED str

        namespace: (namespace) REQUIRED str

        region: (region) REQUIRED str

        session_id: (session_id) REQUIRED str

        source: (source) REQUIRED str

        status: (status) REQUIRED str

        alternate_ips: (alternate_ips) OPTIONAL List[str]

        ams_protocol: (ams_protocol) OPTIONAL List[ModelsPortConfigurationAMS]

        deployment: (deployment) OPTIONAL str

        game_version: (game_version) OPTIONAL str

        image_version: (image_version) OPTIONAL str

        ip: (ip) OPTIONAL str

        pod_name: (pod_name) OPTIONAL str

        port: (port) OPTIONAL int

        ports: (ports) OPTIONAL Dict[str, int]

        protocol: (protocol) OPTIONAL str

        provider: (provider) OPTIONAL str
    """

    # region fields

    custom_attribute: str  # REQUIRED
    is_override_game_version: bool  # REQUIRED
    last_update: str  # REQUIRED
    namespace: str  # REQUIRED
    region: str  # REQUIRED
    session_id: str  # REQUIRED
    source: str  # REQUIRED
    status: str  # REQUIRED
    alternate_ips: List[str]  # OPTIONAL
    ams_protocol: List[ModelsPortConfigurationAMS]  # OPTIONAL
    deployment: str  # OPTIONAL
    game_version: str  # OPTIONAL
    image_version: str  # OPTIONAL
    ip: str  # OPTIONAL
    pod_name: str  # OPTIONAL
    port: int  # OPTIONAL
    ports: Dict[str, int]  # OPTIONAL
    protocol: str  # OPTIONAL
    provider: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_custom_attribute(self, value: str) -> ModelsGameServer:
        self.custom_attribute = value
        return self

    def with_is_override_game_version(self, value: bool) -> ModelsGameServer:
        self.is_override_game_version = value
        return self

    def with_last_update(self, value: str) -> ModelsGameServer:
        self.last_update = value
        return self

    def with_namespace(self, value: str) -> ModelsGameServer:
        self.namespace = value
        return self

    def with_region(self, value: str) -> ModelsGameServer:
        self.region = value
        return self

    def with_session_id(self, value: str) -> ModelsGameServer:
        self.session_id = value
        return self

    def with_source(self, value: str) -> ModelsGameServer:
        self.source = value
        return self

    def with_status(self, value: str) -> ModelsGameServer:
        self.status = value
        return self

    def with_alternate_ips(self, value: List[str]) -> ModelsGameServer:
        self.alternate_ips = value
        return self

    def with_ams_protocol(
        self, value: List[ModelsPortConfigurationAMS]
    ) -> ModelsGameServer:
        self.ams_protocol = value
        return self

    def with_deployment(self, value: str) -> ModelsGameServer:
        self.deployment = value
        return self

    def with_game_version(self, value: str) -> ModelsGameServer:
        self.game_version = value
        return self

    def with_image_version(self, value: str) -> ModelsGameServer:
        self.image_version = value
        return self

    def with_ip(self, value: str) -> ModelsGameServer:
        self.ip = value
        return self

    def with_pod_name(self, value: str) -> ModelsGameServer:
        self.pod_name = value
        return self

    def with_port(self, value: int) -> ModelsGameServer:
        self.port = value
        return self

    def with_ports(self, value: Dict[str, int]) -> ModelsGameServer:
        self.ports = value
        return self

    def with_protocol(self, value: str) -> ModelsGameServer:
        self.protocol = value
        return self

    def with_provider(self, value: str) -> ModelsGameServer:
        self.provider = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "custom_attribute"):
            result["custom_attribute"] = str(self.custom_attribute)
        elif include_empty:
            result["custom_attribute"] = ""
        if hasattr(self, "is_override_game_version"):
            result["is_override_game_version"] = bool(self.is_override_game_version)
        elif include_empty:
            result["is_override_game_version"] = False
        if hasattr(self, "last_update"):
            result["last_update"] = str(self.last_update)
        elif include_empty:
            result["last_update"] = ""
        if hasattr(self, "namespace"):
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "region"):
            result["region"] = str(self.region)
        elif include_empty:
            result["region"] = ""
        if hasattr(self, "session_id"):
            result["session_id"] = str(self.session_id)
        elif include_empty:
            result["session_id"] = ""
        if hasattr(self, "source"):
            result["source"] = str(self.source)
        elif include_empty:
            result["source"] = ""
        if hasattr(self, "status"):
            result["status"] = str(self.status)
        elif include_empty:
            result["status"] = ""
        if hasattr(self, "alternate_ips"):
            result["alternate_ips"] = [str(i0) for i0 in self.alternate_ips]
        elif include_empty:
            result["alternate_ips"] = []
        if hasattr(self, "ams_protocol"):
            result["ams_protocol"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.ams_protocol
            ]
        elif include_empty:
            result["ams_protocol"] = []
        if hasattr(self, "deployment"):
            result["deployment"] = str(self.deployment)
        elif include_empty:
            result["deployment"] = ""
        if hasattr(self, "game_version"):
            result["game_version"] = str(self.game_version)
        elif include_empty:
            result["game_version"] = ""
        if hasattr(self, "image_version"):
            result["image_version"] = str(self.image_version)
        elif include_empty:
            result["image_version"] = ""
        if hasattr(self, "ip"):
            result["ip"] = str(self.ip)
        elif include_empty:
            result["ip"] = ""
        if hasattr(self, "pod_name"):
            result["pod_name"] = str(self.pod_name)
        elif include_empty:
            result["pod_name"] = ""
        if hasattr(self, "port"):
            result["port"] = int(self.port)
        elif include_empty:
            result["port"] = 0
        if hasattr(self, "ports"):
            result["ports"] = {str(k0): int(v0) for k0, v0 in self.ports.items()}
        elif include_empty:
            result["ports"] = {}
        if hasattr(self, "protocol"):
            result["protocol"] = str(self.protocol)
        elif include_empty:
            result["protocol"] = ""
        if hasattr(self, "provider"):
            result["provider"] = str(self.provider)
        elif include_empty:
            result["provider"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        custom_attribute: str,
        is_override_game_version: bool,
        last_update: str,
        namespace: str,
        region: str,
        session_id: str,
        source: str,
        status: str,
        alternate_ips: Optional[List[str]] = None,
        ams_protocol: Optional[List[ModelsPortConfigurationAMS]] = None,
        deployment: Optional[str] = None,
        game_version: Optional[str] = None,
        image_version: Optional[str] = None,
        ip: Optional[str] = None,
        pod_name: Optional[str] = None,
        port: Optional[int] = None,
        ports: Optional[Dict[str, int]] = None,
        protocol: Optional[str] = None,
        provider: Optional[str] = None,
        **kwargs,
    ) -> ModelsGameServer:
        instance = cls()
        instance.custom_attribute = custom_attribute
        instance.is_override_game_version = is_override_game_version
        instance.last_update = last_update
        instance.namespace = namespace
        instance.region = region
        instance.session_id = session_id
        instance.source = source
        instance.status = status
        if alternate_ips is not None:
            instance.alternate_ips = alternate_ips
        if ams_protocol is not None:
            instance.ams_protocol = ams_protocol
        if deployment is not None:
            instance.deployment = deployment
        if game_version is not None:
            instance.game_version = game_version
        if image_version is not None:
            instance.image_version = image_version
        if ip is not None:
            instance.ip = ip
        if pod_name is not None:
            instance.pod_name = pod_name
        if port is not None:
            instance.port = port
        if ports is not None:
            instance.ports = ports
        if protocol is not None:
            instance.protocol = protocol
        if provider is not None:
            instance.provider = provider
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelsGameServer:
        instance = cls()
        if not dict_:
            return instance
        if "custom_attribute" in dict_ and dict_["custom_attribute"] is not None:
            instance.custom_attribute = str(dict_["custom_attribute"])
        elif include_empty:
            instance.custom_attribute = ""
        if (
            "is_override_game_version" in dict_
            and dict_["is_override_game_version"] is not None
        ):
            instance.is_override_game_version = bool(dict_["is_override_game_version"])
        elif include_empty:
            instance.is_override_game_version = False
        if "last_update" in dict_ and dict_["last_update"] is not None:
            instance.last_update = str(dict_["last_update"])
        elif include_empty:
            instance.last_update = ""
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "region" in dict_ and dict_["region"] is not None:
            instance.region = str(dict_["region"])
        elif include_empty:
            instance.region = ""
        if "session_id" in dict_ and dict_["session_id"] is not None:
            instance.session_id = str(dict_["session_id"])
        elif include_empty:
            instance.session_id = ""
        if "source" in dict_ and dict_["source"] is not None:
            instance.source = str(dict_["source"])
        elif include_empty:
            instance.source = ""
        if "status" in dict_ and dict_["status"] is not None:
            instance.status = str(dict_["status"])
        elif include_empty:
            instance.status = ""
        if "alternate_ips" in dict_ and dict_["alternate_ips"] is not None:
            instance.alternate_ips = [str(i0) for i0 in dict_["alternate_ips"]]
        elif include_empty:
            instance.alternate_ips = []
        if "ams_protocol" in dict_ and dict_["ams_protocol"] is not None:
            instance.ams_protocol = [
                ModelsPortConfigurationAMS.create_from_dict(
                    i0, include_empty=include_empty
                )
                for i0 in dict_["ams_protocol"]
            ]
        elif include_empty:
            instance.ams_protocol = []
        if "deployment" in dict_ and dict_["deployment"] is not None:
            instance.deployment = str(dict_["deployment"])
        elif include_empty:
            instance.deployment = ""
        if "game_version" in dict_ and dict_["game_version"] is not None:
            instance.game_version = str(dict_["game_version"])
        elif include_empty:
            instance.game_version = ""
        if "image_version" in dict_ and dict_["image_version"] is not None:
            instance.image_version = str(dict_["image_version"])
        elif include_empty:
            instance.image_version = ""
        if "ip" in dict_ and dict_["ip"] is not None:
            instance.ip = str(dict_["ip"])
        elif include_empty:
            instance.ip = ""
        if "pod_name" in dict_ and dict_["pod_name"] is not None:
            instance.pod_name = str(dict_["pod_name"])
        elif include_empty:
            instance.pod_name = ""
        if "port" in dict_ and dict_["port"] is not None:
            instance.port = int(dict_["port"])
        elif include_empty:
            instance.port = 0
        if "ports" in dict_ and dict_["ports"] is not None:
            instance.ports = {str(k0): int(v0) for k0, v0 in dict_["ports"].items()}
        elif include_empty:
            instance.ports = {}
        if "protocol" in dict_ and dict_["protocol"] is not None:
            instance.protocol = str(dict_["protocol"])
        elif include_empty:
            instance.protocol = ""
        if "provider" in dict_ and dict_["provider"] is not None:
            instance.provider = str(dict_["provider"])
        elif include_empty:
            instance.provider = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelsGameServer]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelsGameServer]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[ModelsGameServer, List[ModelsGameServer], Dict[Any, ModelsGameServer]]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "custom_attribute": "custom_attribute",
            "is_override_game_version": "is_override_game_version",
            "last_update": "last_update",
            "namespace": "namespace",
            "region": "region",
            "session_id": "session_id",
            "source": "source",
            "status": "status",
            "alternate_ips": "alternate_ips",
            "ams_protocol": "ams_protocol",
            "deployment": "deployment",
            "game_version": "game_version",
            "image_version": "image_version",
            "ip": "ip",
            "pod_name": "pod_name",
            "port": "port",
            "ports": "ports",
            "protocol": "protocol",
            "provider": "provider",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "custom_attribute": True,
            "is_override_game_version": True,
            "last_update": True,
            "namespace": True,
            "region": True,
            "session_id": True,
            "source": True,
            "status": True,
            "alternate_ips": False,
            "ams_protocol": False,
            "deployment": False,
            "game_version": False,
            "image_version": False,
            "ip": False,
            "pod_name": False,
            "port": False,
            "ports": False,
            "protocol": False,
            "provider": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the AccelByte Gaming Services Ugc Service."""

__version__ = "2.19.4"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .get_followed_content import GetFollowedContent
from .get_followed_users import GetFollowedUsers
from .get_public_followers import GetPublicFollowers
from .get_public_following import GetPublicFollowing
from .update_user_follow_status import UpdateUserFollowStatus



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Match Service V2

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class ConfigEnvironmentVariable(Model):
    """Config environment variable (config.EnvironmentVariable)

    Properties:
        name: (name) REQUIRED str

        actual_value: (actualValue) OPTIONAL str

        default_value: (defaultValue) OPTIONAL str

        description: (description) OPTIONAL str
    """

    # region fields

    name: str  # REQUIRED
    actual_value: str  # OPTIONAL
    default_value: str  # OPTIONAL
    description: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_name(self, value: str) -> ConfigEnvironmentVariable:
        self.name = value
        return self

    def with_actual_value(self, value: str) -> ConfigEnvironmentVariable:
        self.actual_value = value
        return self

    def with_default_value(self, value: str) -> ConfigEnvironmentVariable:
        self.default_value = value
        return self

    def with_description(self, value: str) -> ConfigEnvironmentVariable:
        self.description = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "name"):
            result["name"] = str(self.name)
        elif include_empty:
            result["name"] = ""
        if hasattr(self, "actual_value"):
            result["actualValue"] = str(self.actual_value)
        elif include_empty:
            result["actualValue"] = ""
        if hasattr(self, "default_value"):
            result["defaultValue"] = str(self.default_value)
        elif include_empty:
            result["defaultValue"] = ""
        if hasattr(self, "description"):
            result["description"] = str(self.description)
        elif include_empty:
            result["description"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        name: str,
        actual_value: Optional[str] = None,
        default_value: Optional[str] = None,
        description: Optional[str] = None,
        **kwargs,
    ) -> ConfigEnvironmentVariable:
        instance = cls()
        instance.name = name
        if actual_value is not None:
            instance.actual_value = actual_value
        if default_value is not None:
            instance.default_value = default_value
        if description is not None:
            instance.description = description
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ConfigEnvironmentVariable:
        instance = cls()
        if not dict_:
            return instance
        if "name" in dict_ and dict_["name"] is not None:
            instance.name = str(dict_["name"])
        elif include_empty:
            instance.name = ""
        if "actualValue" in dict_ and dict_["actualValue"] is not None:
            instance.actual_value = str(dict_["actualValue"])
        elif include_empty:
            instance.actual_value = ""
        if "defaultValue" in dict_ and dict_["defaultValue"] is not None:
            instance.default_value = str(dict_["defaultValue"])
        elif include_empty:
            instance.default_value = ""
        if "description" in dict_ and dict_["description"] is not None:
            instance.description = str(dict_["description"])
        elif include_empty:
            instance.description = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ConfigEnvironmentVariable]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ConfigEnvironmentVariable]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ConfigEnvironmentVariable,
        List[ConfigEnvironmentVariable],
        Dict[Any, ConfigEnvironmentVariable],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "name": "name",
            "actualValue": "actual_value",
            "defaultValue": "default_value",
            "description": "description",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "name": True,
            "actualValue": False,
            "defaultValue": False,
            "description": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Platform Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model
from ....core import StrEnum


class ItemIdentityTypeEnum(StrEnum):
    ITEM_ID = "ITEM_ID"
    ITEM_SKU = "ITEM_SKU"


class ConsumeItem(Model):
    """Consume item (ConsumeItem)

    Properties:
        ext_item_def_id: (extItemDefId) OPTIONAL str

        ext_item_id: (extItemId) OPTIONAL str

        item_identity: (itemIdentity) OPTIONAL str

        item_identity_type: (itemIdentityType) OPTIONAL Union[str, ItemIdentityTypeEnum]
    """

    # region fields

    ext_item_def_id: str  # OPTIONAL
    ext_item_id: str  # OPTIONAL
    item_identity: str  # OPTIONAL
    item_identity_type: Union[str, ItemIdentityTypeEnum]  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_ext_item_def_id(self, value: str) -> ConsumeItem:
        self.ext_item_def_id = value
        return self

    def with_ext_item_id(self, value: str) -> ConsumeItem:
        self.ext_item_id = value
        return self

    def with_item_identity(self, value: str) -> ConsumeItem:
        self.item_identity = value
        return self

    def with_item_identity_type(
        self, value: Union[str, ItemIdentityTypeEnum]
    ) -> ConsumeItem:
        self.item_identity_type = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "ext_item_def_id"):
            result["extItemDefId"] = str(self.ext_item_def_id)
        elif include_empty:
            result["extItemDefId"] = ""
        if hasattr(self, "ext_item_id"):
            result["extItemId"] = str(self.ext_item_id)
        elif include_empty:
            result["extItemId"] = ""
        if hasattr(self, "item_identity"):
            result["itemIdentity"] = str(self.item_identity)
        elif include_empty:
            result["itemIdentity"] = ""
        if hasattr(self, "item_identity_type"):
            result["itemIdentityType"] = str(self.item_identity_type)
        elif include_empty:
            result["itemIdentityType"] = Union[str, ItemIdentityTypeEnum]()
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        ext_item_def_id: Optional[str] = None,
        ext_item_id: Optional[str] = None,
        item_identity: Optional[str] = None,
        item_identity_type: Optional[Union[str, ItemIdentityTypeEnum]] = None,
        **kwargs,
    ) -> ConsumeItem:
        instance = cls()
        if ext_item_def_id is not None:
            instance.ext_item_def_id = ext_item_def_id
        if ext_item_id is not None:
            instance.ext_item_id = ext_item_id
        if item_identity is not None:
            instance.item_identity = item_identity
        if item_identity_type is not None:
            instance.item_identity_type = item_identity_type
        return instance

    @classmethod
    def create_from_dict(cls, dict_: dict, include_empty: bool = False) -> ConsumeItem:
        instance = cls()
        if not dict_:
            return instance
        if "extItemDefId" in dict_ and dict_["extItemDefId"] is not None:
            instance.ext_item_def_id = str(dict_["extItemDefId"])
        elif include_empty:
            instance.ext_item_def_id = ""
        if "extItemId" in dict_ and dict_["extItemId"] is not None:
            instance.ext_item_id = str(dict_["extItemId"])
        elif include_empty:
            instance.ext_item_id = ""
        if "itemIdentity" in dict_ and dict_["itemIdentity"] is not None:
            instance.item_identity = str(dict_["itemIdentity"])
        elif include_empty:
            instance.item_identity = ""
        if "itemIdentityType" in dict_ and dict_["itemIdentityType"] is not None:
            instance.item_identity_type = str(dict_["itemIdentityType"])
        elif include_empty:
            instance.item_identity_type = Union[str, ItemIdentityTypeEnum]()
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ConsumeItem]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ConsumeItem]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[ConsumeItem, List[ConsumeItem], Dict[Any, ConsumeItem]]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "extItemDefId": "ext_item_def_id",
            "extItemId": "ext_item_id",
            "itemIdentity": "item_identity",
            "itemIdentityType": "item_identity_type",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "extItemDefId": False,
            "extItemId": False,
            "itemIdentity": False,
            "itemIdentityType": False,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "itemIdentityType": ["ITEM_ID", "ITEM_SKU"],
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Platform Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.sub_item_available_price import SubItemAvailablePrice


class AvailablePrice(Model):
    """Available price (AvailablePrice)

    Properties:
        currency_code: (currencyCode) REQUIRED str

        currency_namespace: (currencyNamespace) REQUIRED str

        discounted_price: (discountedPrice) REQUIRED int

        price: (price) REQUIRED int

        discount_expire_at: (discountExpireAt) OPTIONAL str

        discount_purchase_at: (discountPurchaseAt) OPTIONAL str

        expire_at: (expireAt) OPTIONAL str

        price_details: (priceDetails) OPTIONAL List[SubItemAvailablePrice]

        purchase_at: (purchaseAt) OPTIONAL str
    """

    # region fields

    currency_code: str  # REQUIRED
    currency_namespace: str  # REQUIRED
    discounted_price: int  # REQUIRED
    price: int  # REQUIRED
    discount_expire_at: str  # OPTIONAL
    discount_purchase_at: str  # OPTIONAL
    expire_at: str  # OPTIONAL
    price_details: List[SubItemAvailablePrice]  # OPTIONAL
    purchase_at: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_currency_code(self, value: str) -> AvailablePrice:
        self.currency_code = value
        return self

    def with_currency_namespace(self, value: str) -> AvailablePrice:
        self.currency_namespace = value
        return self

    def with_discounted_price(self, value: int) -> AvailablePrice:
        self.discounted_price = value
        return self

    def with_price(self, value: int) -> AvailablePrice:
        self.price = value
        return self

    def with_discount_expire_at(self, value: str) -> AvailablePrice:
        self.discount_expire_at = value
        return self

    def with_discount_purchase_at(self, value: str) -> AvailablePrice:
        self.discount_purchase_at = value
        return self

    def with_expire_at(self, value: str) -> AvailablePrice:
        self.expire_at = value
        return self

    def with_price_details(self, value: List[SubItemAvailablePrice]) -> AvailablePrice:
        self.price_details = value
        return self

    def with_purchase_at(self, value: str) -> AvailablePrice:
        self.purchase_at = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "currency_code"):
            result["currencyCode"] = str(self.currency_code)
        elif include_empty:
            result["currencyCode"] = ""
        if hasattr(self, "currency_namespace"):
            result["currencyNamespace"] = str(self.currency_namespace)
        elif include_empty:
            result["currencyNamespace"] = ""
        if hasattr(self, "discounted_price"):
            result["discountedPrice"] = int(self.discounted_price)
        elif include_empty:
            result["discountedPrice"] = 0
        if hasattr(self, "price"):
            result["price"] = int(self.price)
        elif include_empty:
            result["price"] = 0
        if hasattr(self, "discount_expire_at"):
            result["discountExpireAt"] = str(self.discount_expire_at)
        elif include_empty:
            result["discountExpireAt"] = ""
        if hasattr(self, "discount_purchase_at"):
            result["discountPurchaseAt"] = str(self.discount_purchase_at)
        elif include_empty:
            result["discountPurchaseAt"] = ""
        if hasattr(self, "expire_at"):
            result["expireAt"] = str(self.expire_at)
        elif include_empty:
            result["expireAt"] = ""
        if hasattr(self, "price_details"):
            result["priceDetails"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.price_details
            ]
        elif include_empty:
            result["priceDetails"] = []
        if hasattr(self, "purchase_at"):
            result["purchaseAt"] = str(self.purchase_at)
        elif include_empty:
            result["purchaseAt"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        currency_code: str,
        currency_namespace: str,
        discounted_price: int,
        price: int,
        discount_expire_at: Optional[str] = None,
        discount_purchase_at: Optional[str] = None,
        expire_at: Optional[str] = None,
        price_details: Optional[List[SubItemAvailablePrice]] = None,
        purchase_at: Optional[str] = None,
        **kwargs,
    ) -> AvailablePrice:
        instance = cls()
        instance.currency_code = currency_code
        instance.currency_namespace = currency_namespace
        instance.discounted_price = discounted_price
        instance.price = price
        if discount_expire_at is not None:
            instance.discount_expire_at = discount_expire_at
        if discount_purchase_at is not None:
            instance.discount_purchase_at = discount_purchase_at
        if expire_at is not None:
            instance.expire_at = expire_at
        if price_details is not None:
            instance.price_details = price_details
        if purchase_at is not None:
            instance.purchase_at = purchase_at
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> AvailablePrice:
        instance = cls()
        if not dict_:
            return instance
        if "currencyCode" in dict_ and dict_["currencyCode"] is not None:
            instance.currency_code = str(dict_["currencyCode"])
        elif include_empty:
            instance.currency_code = ""
        if "currencyNamespace" in dict_ and dict_["currencyNamespace"] is not None:
            instance.currency_namespace = str(dict_["currencyNamespace"])
        elif include_empty:
            instance.currency_namespace = ""
        if "discountedPrice" in dict_ and dict_["discountedPrice"] is not None:
            instance.discounted_price = int(dict_["discountedPrice"])
        elif include_empty:
            instance.discounted_price = 0
        if "price" in dict_ and dict_["price"] is not None:
            instance.price = int(dict_["price"])
        elif include_empty:
            instance.price = 0
        if "discountExpireAt" in dict_ and dict_["discountExpireAt"] is not None:
            instance.discount_expire_at = str(dict_["discountExpireAt"])
        elif include_empty:
            instance.discount_expire_at = ""
        if "discountPurchaseAt" in dict_ and dict_["discountPurchaseAt"] is not None:
            instance.discount_purchase_at = str(dict_["discountPurchaseAt"])
        elif include_empty:
            instance.discount_purchase_at = ""
        if "expireAt" in dict_ and dict_["expireAt"] is not None:
            instance.expire_at = str(dict_["expireAt"])
        elif include_empty:
            instance.expire_at = ""
        if "priceDetails" in dict_ and dict_["priceDetails"] is not None:
            instance.price_details = [
                SubItemAvailablePrice.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["priceDetails"]
            ]
        elif include_empty:
            instance.price_details = []
        if "purchaseAt" in dict_ and dict_["purchaseAt"] is not None:
            instance.purchase_at = str(dict_["purchaseAt"])
        elif include_empty:
            instance.purchase_at = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, AvailablePrice]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[AvailablePrice]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[AvailablePrice, List[AvailablePrice], Dict[Any, AvailablePrice]]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "currencyCode": "currency_code",
            "currencyNamespace": "currency_namespace",
            "discountedPrice": "discounted_price",
            "price": "price",
            "discountExpireAt": "discount_expire_at",
            "discountPurchaseAt": "discount_purchase_at",
            "expireAt": "expire_at",
            "priceDetails": "price_details",
            "purchaseAt": "purchase_at",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "currencyCode": True,
            "currencyNamespace": True,
            "discountedPrice": True,
            "price": True,
            "discountExpireAt": False,
            "discountPurchaseAt": False,
            "expireAt": False,
            "priceDetails": False,
            "purchaseAt": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the AccelByte Gaming Services Social Service."""

__version__ = "2.11.3"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from ._game_profile import get_profile
from ._game_profile import get_profile_async
from ._game_profile import get_user_profiles
from ._game_profile import get_user_profiles_async
from ._game_profile import public_create_profile
from ._game_profile import public_create_profile_async
from ._game_profile import public_delete_profile
from ._game_profile import public_delete_profile_async
from ._game_profile import public_get_profile
from ._game_profile import public_get_profile_async
from ._game_profile import public_get_profile_attribute
from ._game_profile import public_get_profile_attribute_async
from ._game_profile import public_get_user_game_profiles
from ._game_profile import public_get_user_game_profiles_async
from ._game_profile import public_get_user_profiles
from ._game_profile import public_get_user_profiles_async
from ._game_profile import public_update_attribute
from ._game_profile import public_update_attribute_async
from ._game_profile import public_update_profile
from ._game_profile import public_update_profile_async

from ._global_statistic import get_global_stat_item_by_stat_code
from ._global_statistic import get_global_stat_item_by_stat_code_async
from ._global_statistic import get_global_stat_item_by_stat_code_1
from ._global_statistic import get_global_stat_item_by_stat_code_1_async
from ._global_statistic import get_global_stat_items
from ._global_statistic import get_global_stat_items_async
from ._global_statistic import get_global_stat_items_1
from ._global_statistic import get_global_stat_items_1_async

from ._slot import get_slot_data
from ._slot import get_slot_data_async
from ._slot import get_user_namespace_slots
from ._slot import get_user_namespace_slots_async
from ._slot import public_create_user_namespace_slot
from ._slot import public_create_user_namespace_slot_async
from ._slot import public_delete_user_namespace_slot
from ._slot import public_delete_user_namespace_slot_async
from ._slot import public_get_slot_data
from ._slot import public_get_slot_data_async
from ._slot import public_get_user_namespace_slots
from ._slot import public_get_user_namespace_slots_async
from ._slot import public_update_user_namespace_slot
from ._slot import public_update_user_namespace_slot_async
from ._slot import public_update_user_namespace_slot_metadata
from ._slot import public_update_user_namespace_slot_metadata_async

from ._slot_config import delete_namespace_slot_config
from ._slot_config import delete_namespace_slot_config_async
from ._slot_config import delete_user_slot_config
from ._slot_config import delete_user_slot_config_async
from ._slot_config import get_namespace_slot_config
from ._slot_config import get_namespace_slot_config_async
from ._slot_config import get_user_slot_config
from ._slot_config import get_user_slot_config_async
from ._slot_config import update_namespace_slot_config
from ._slot_config import update_namespace_slot_config_async
from ._slot_config import update_user_slot_config
from ._slot_config import update_user_slot_config_async

from ._stat_configuration import create_stat
from ._stat_configuration import create_stat_async
from ._stat_configuration import create_stat_1
from ._stat_configuration import create_stat_1_async
from ._stat_configuration import delete_stat
from ._stat_configuration import delete_stat_async
from ._stat_configuration import delete_tied_stat
from ._stat_configuration import delete_tied_stat_async
from ._stat_configuration import export_stats
from ._stat_configuration import export_stats_async
from ._stat_configuration import get_stat
from ._stat_configuration import get_stat_async
from ._stat_configuration import get_stats
from ._stat_configuration import get_stats_async
from ._stat_configuration import import_stats
from ._stat_configuration import import_stats_async
from ._stat_configuration import query_stats
from ._stat_configuration import query_stats_async
from ._stat_configuration import update_stat
from ._stat_configuration import update_stat_async

from ._stat_cycle_configuration import bulk_add_stats
from ._stat_cycle_configuration import bulk_add_stats_async
from ._stat_cycle_configuration import bulk_get_stat_cycle
from ._stat_cycle_configuration import bulk_get_stat_cycle_async
from ._stat_cycle_configuration import bulk_get_stat_cycle_1
from ._stat_cycle_configuration import bulk_get_stat_cycle_1_async
from ._stat_cycle_configuration import create_stat_cycle
from ._stat_cycle_configuration import create_stat_cycle_async
from ._stat_cycle_configuration import delete_stat_cycle
from ._stat_cycle_configuration import delete_stat_cycle_async
from ._stat_cycle_configuration import export_stat_cycle
from ._stat_cycle_configuration import export_stat_cycle_async
from ._stat_cycle_configuration import get_stat_cycle
from ._stat_cycle_configuration import get_stat_cycle_async
from ._stat_cycle_configuration import get_stat_cycle_1
from ._stat_cycle_configuration import get_stat_cycle_1_async
from ._stat_cycle_configuration import get_stat_cycles
from ._stat_cycle_configuration import get_stat_cycles_async
from ._stat_cycle_configuration import get_stat_cycles_1
from ._stat_cycle_configuration import get_stat_cycles_1_async
from ._stat_cycle_configuration import import_stat_cycle
from ._stat_cycle_configuration import import_stat_cycle_async
from ._stat_cycle_configuration import stop_stat_cycle
from ._stat_cycle_configuration import stop_stat_cycle_async
from ._stat_cycle_configuration import update_stat_cycle
from ._stat_cycle_configuration import update_stat_cycle_async

from ._user_statistic import admin_list_users_stat_items
from ._user_statistic import admin_list_users_stat_items_async
from ._user_statistic import bulk_create_user_stat_items
from ._user_statistic import bulk_create_user_stat_items_async
from ._user_statistic import bulk_fetch_or_default_stat_items
from ._user_statistic import bulk_fetch_or_default_stat_items_async
from ._user_statistic import bulk_fetch_or_default_stat_items_1
from ._user_statistic import bulk_fetch_or_default_stat_items_1_async
from ._user_statistic import bulk_fetch_stat_items
from ._user_statistic import bulk_fetch_stat_items_async
from ._user_statistic import bulk_fetch_stat_items_1
from ._user_statistic import bulk_fetch_stat_items_1_async
from ._user_statistic import bulk_inc_user_stat_item
from ._user_statistic import bulk_inc_user_stat_item_async
from ._user_statistic import bulk_inc_user_stat_item_1
from ._user_statistic import bulk_inc_user_stat_item_1_async
from ._user_statistic import bulk_inc_user_stat_item_value
from ._user_statistic import bulk_inc_user_stat_item_value_async
from ._user_statistic import bulk_inc_user_stat_item_value_1
from ._user_statistic import bulk_inc_user_stat_item_value_1_async
from ._user_statistic import bulk_inc_user_stat_item_value_2
from ._user_statistic import bulk_inc_user_stat_item_value_2_async
from ._user_statistic import bulk_reset_user_stat_item
from ._user_statistic import bulk_reset_user_stat_item_async
from ._user_statistic import bulk_reset_user_stat_item_1
from ._user_statistic import bulk_reset_user_stat_item_1_async
from ._user_statistic import bulk_reset_user_stat_item_2
from ._user_statistic import bulk_reset_user_stat_item_2_async
from ._user_statistic import bulk_reset_user_stat_item_3
from ._user_statistic import bulk_reset_user_stat_item_3_async
from ._user_statistic import bulk_reset_user_stat_item_values
from ._user_statistic import bulk_reset_user_stat_item_values_async
from ._user_statistic import bulk_update_user_stat_item
from ._user_statistic import bulk_update_user_stat_item_async
from ._user_statistic import bulk_update_user_stat_item_1
from ._user_statistic import bulk_update_user_stat_item_1_async
from ._user_statistic import bulk_update_user_stat_item_2
from ._user_statistic import bulk_update_user_stat_item_2_async
from ._user_statistic import bulk_update_user_stat_item_v2
from ._user_statistic import bulk_update_user_stat_item_v2_async
from ._user_statistic import create_user_stat_item
from ._user_statistic import create_user_stat_item_async
from ._user_statistic import delete_user_stat_items
from ._user_statistic import delete_user_stat_items_async
from ._user_statistic import delete_user_stat_items_1
from ._user_statistic import delete_user_stat_items_1_async
from ._user_statistic import delete_user_stat_items_2
from ._user_statistic import delete_user_stat_items_2_async
from ._user_statistic import get_stat_items
from ._user_statistic import get_stat_items_async
from ._user_statistic import get_user_stat_items
from ._user_statistic import get_user_stat_items_async
from ._user_statistic import inc_user_stat_item_value
from ._user_statistic import inc_user_stat_item_value_async
from ._user_statistic import public_bulk_create_user_stat_items
from ._user_statistic import public_bulk_create_user_stat_items_async
from ._user_statistic import public_bulk_inc_user_stat_item
from ._user_statistic import public_bulk_inc_user_stat_item_async
from ._user_statistic import public_bulk_inc_user_stat_item_1
from ._user_statistic import public_bulk_inc_user_stat_item_1_async
from ._user_statistic import public_bulk_inc_user_stat_item_value
from ._user_statistic import public_bulk_inc_user_stat_item_value_async
from ._user_statistic import public_create_user_stat_item
from ._user_statistic import public_create_user_stat_item_async
from ._user_statistic import public_inc_user_stat_item
from ._user_statistic import public_inc_user_stat_item_async
from ._user_statistic import public_inc_user_stat_item_value
from ._user_statistic import public_inc_user_stat_item_value_async
from ._user_statistic import public_list_all_my_stat_items
from ._user_statistic import public_list_all_my_stat_items_async
from ._user_statistic import public_list_my_stat_items
from ._user_statistic import public_list_my_stat_items_async
from ._user_statistic import public_query_user_stat_items
from ._user_statistic import public_query_user_stat_items_async
from ._user_statistic import public_query_user_stat_items_1
from ._user_statistic import public_query_user_stat_items_1_async
from ._user_statistic import public_query_user_stat_items_2
from ._user_statistic import public_query_user_stat_items_2_async
from ._user_statistic import reset_user_stat_item_value
from ._user_statistic import reset_user_stat_item_value_async
from ._user_statistic import reset_user_stat_item_value_1
from ._user_statistic import reset_user_stat_item_value_1_async
from ._user_statistic import update_user_stat_item_value
from ._user_statistic import update_user_stat_item_value_async
from ._user_statistic import update_user_stat_item_value_1
from ._user_statistic import update_user_stat_item_value_1_async

from ._user_statistic_cycle import get_user_stat_cycle_items
from ._user_statistic_cycle import get_user_stat_cycle_items_async
from ._user_statistic_cycle import get_user_stat_cycle_items_1
from ._user_statistic_cycle import get_user_stat_cycle_items_1_async
from ._user_statistic_cycle import public_list_my_stat_cycle_items
from ._user_statistic_cycle import public_list_my_stat_cycle_items_async



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

# AccelByte Gaming Services Social Service

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from .....core import Operation
from .....core import HeaderStr
from .....core import HttpResponse

from ...models import ErrorEntity
from ...models import UserStatItemPagingSlicedResult
from ...models import ValidationErrorEntity


class PublicListMyStatItems(Operation):
    """Public list user's statItems (publicListMyStatItems)

    Public list all statItems by pagination.
    Other detail info:

      *  Required permission : resource="NAMESPACE:{namespace}:USER:{userId}:STATITEM", action=2 (READ)
      *  Returns : stat items

    Required Permission(s):
        - NAMESPACE:{namespace}:USER:{userId}:STATITEM [READ]

    Properties:
        url: /social/v1/public/namespaces/{namespace}/users/me/statitems

        method: GET

        tags: ["UserStatistic"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        sort_by: (sortBy) OPTIONAL str in query

        stat_codes: (statCodes) OPTIONAL List[str] in query

        tags: (tags) OPTIONAL List[str] in query

    Responses:
        200: OK - UserStatItemPagingSlicedResult (successful operation)

        401: Unauthorized - ErrorEntity (20001: Unauthorized)

        403: Forbidden - ErrorEntity (20013: insufficient permission)

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)

        500: Internal Server Error - ErrorEntity (20000: Internal server error)
    """

    # region fields

    _url: str = "/social/v1/public/namespaces/{namespace}/users/me/statitems"
    _method: str = "GET"
    _consumes: List[str] = []
    _produces: List[str] = ["application/json"]
    _securities: List[List[str]] = [["BEARER_AUTH"], ["BEARER_AUTH"]]
    _location_query: str = None

    namespace: str  # REQUIRED in [path]
    limit: int  # OPTIONAL in [query]
    offset: int  # OPTIONAL in [query]
    sort_by: str  # OPTIONAL in [query]
    stat_codes: List[str]  # OPTIONAL in [query]
    tags: List[str]  # OPTIONAL in [query]

    # endregion fields

    # region properties

    @property
    def url(self) -> str:
        return self._url

    @property
    def method(self) -> str:
        return self._method

    @property
    def consumes(self) -> List[str]:
        return self._consumes

    @property
    def produces(self) -> List[str]:
        return self._produces

    @property
    def securities(self) -> List[List[str]]:
        return self._securities

    @property
    def location_query(self) -> str:
        return self._location_query

    # endregion properties

    # region get methods

    # endregion get methods

    # region get_x_params methods

    def get_all_params(self) -> dict:
        return {
            "path": self.get_path_params(),
            "query": self.get_query_params(),
        }

    def get_path_params(self) -> dict:
        result = {}
        if hasattr(self, "namespace"):
            result["namespace"] = self.namespace
        return result

    def get_query_params(self) -> dict:
        result = {}
        if hasattr(self, "limit"):
            result["limit"] = self.limit
        if hasattr(self, "offset"):
            result["offset"] = self.offset
        if hasattr(self, "sort_by"):
            result["sortBy"] = self.sort_by
        if hasattr(self, "stat_codes"):
            result["statCodes"] = self.stat_codes
        if hasattr(self, "tags"):
            result["tags"] = self.tags
        return result

    # endregion get_x_params methods

    # region is/has methods

    # endregion is/has methods

    # region with_x methods

    def with_namespace(self, value: str) -> PublicListMyStatItems:
        self.namespace = value
        return self

    def with_limit(self, value: int) -> PublicListMyStatItems:
        self.limit = value
        return self

    def with_offset(self, value: int) -> PublicListMyStatItems:
        self.offset = value
        return self

    def with_sort_by(self, value: str) -> PublicListMyStatItems:
        self.sort_by = value
        return self

    def with_stat_codes(self, value: List[str]) -> PublicListMyStatItems:
        self.stat_codes = value
        return self

    def with_tags(self, value: List[str]) -> PublicListMyStatItems:
        self.tags = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "namespace") and self.namespace:
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "limit") and self.limit:
            result["limit"] = int(self.limit)
        elif include_empty:
            result["limit"] = 0
        if hasattr(self, "offset") and self.offset:
            result["offset"] = int(self.offset)
        elif include_empty:
            result["offset"] = 0
        if hasattr(self, "sort_by") and self.sort_by:
            result["sortBy"] = str(self.sort_by)
        elif include_empty:
            result["sortBy"] = ""
        if hasattr(self, "stat_codes") and self.stat_codes:
            result["statCodes"] = [str(i0) for i0 in self.stat_codes]
        elif include_empty:
            result["statCodes"] = []
        if hasattr(self, "tags") and self.tags:
            result["tags"] = [str(i0) for i0 in self.tags]
        elif include_empty:
            result["tags"] = []
        return result

    # endregion to methods

    # region response methods

    # noinspection PyMethodMayBeStatic
    def parse_response(
        self, code: int, content_type: str, content: Any
    ) -> Tuple[
        Union[None, UserStatItemPagingSlicedResult],
        Union[None, ErrorEntity, HttpResponse, ValidationErrorEntity],
    ]:
        """Parse the given response.

        200: OK - UserStatItemPagingSlicedResult (successful operation)

        401: Unauthorized - ErrorEntity (20001: Unauthorized)

        403: Forbidden - ErrorEntity (20013: insufficient permission)

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)

        500: Internal Server Error - ErrorEntity (20000: Internal server error)

        ---: HttpResponse (Undocumented Response)

        ---: HttpResponse (Unexpected Content-Type Error)

        ---: HttpResponse (Unhandled Error)
        """
        pre_processed_response, error = self.pre_process_response(
            code=code, content_type=content_type, content=content
        )
        if error is not None:
            return None, None if error.is_no_content() else error
        code, content_type, content = pre_processed_response

        if code == 200:
            return UserStatItemPagingSlicedResult.create_from_dict(content), None
        if code == 401:
            return None, ErrorEntity.create_from_dict(content)
        if code == 403:
            return None, ErrorEntity.create_from_dict(content)
        if code == 422:
            return None, ValidationErrorEntity.create_from_dict(content)
        if code == 500:
            return None, ErrorEntity.create_from_dict(content)

        return self.handle_undocumented_response(
            code=code, content_type=content_type, content=content
        )

    # endregion response methods

    # region static methods

    @classmethod
    def create(
        cls,
        namespace: str,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        sort_by: Optional[str] = None,
        stat_codes: Optional[List[str]] = None,
        tags: Optional[List[str]] = None,
        **kwargs,
    ) -> PublicListMyStatItems:
        instance = cls()
        instance.namespace = namespace
        if limit is not None:
            instance.limit = limit
        if offset is not None:
            instance.offset = offset
        if sort_by is not None:
            instance.sort_by = sort_by
        if stat_codes is not None:
            instance.stat_codes = stat_codes
        if tags is not None:
            instance.tags = tags
        if x_flight_id := kwargs.get("x_flight_id", None):
            instance.x_flight_id = x_flight_id
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> PublicListMyStatItems:
        instance = cls()
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "limit" in dict_ and dict_["limit"] is not None:
            instance.limit = int(dict_["limit"])
        elif include_empty:
            instance.limit = 0
        if "offset" in dict_ and dict_["offset"] is not None:
            instance.offset = int(dict_["offset"])
        elif include_empty:
            instance.offset = 0
        if "sortBy" in dict_ and dict_["sortBy"] is not None:
            instance.sort_by = str(dict_["sortBy"])
        elif include_empty:
            instance.sort_by = ""
        if "statCodes" in dict_ and dict_["statCodes"] is not None:
            instance.stat_codes = [str(i0) for i0 in dict_["statCodes"]]
        elif include_empty:
            instance.stat_codes = []
        if "tags" in dict_ and dict_["tags"] is not None:
            instance.tags = [str(i0) for i0 in dict_["tags"]]
        elif include_empty:
            instance.tags = []
        return instance

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "namespace": "namespace",
            "limit": "limit",
            "offset": "offset",
            "sortBy": "sort_by",
            "statCodes": "stat_codes",
            "tags": "tags",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "namespace": True,
            "limit": False,
            "offset": False,
            "sortBy": False,
            "statCodes": False,
            "tags": False,
        }

    @staticmethod
    def get_collection_format_map() -> Dict[str, Union[None, str]]:
        return {
            "statCodes": "multi",  # in query
            "tags": "multi",  # in query
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import DtoServiceConfigurationUpdateRequest
from ..models import DtoServicesConfigurationResponse
from ..models import ResponseError

from ..operations.configuration import AdminGetServicesConfiguration
from ..operations.configuration import AdminResetServicesConfiguration
from ..operations.configuration import AdminUpdateServicesConfiguration
from ..operations.configuration import DeleteAdminEmailConfiguration
from ..operations.configuration import GetAdminEmailConfiguration
from ..operations.configuration import SaveAdminEmailConfiguration
from ..operations.configuration import UpdateAdminEmailConfiguration


@same_doc_as(AdminGetServicesConfiguration)
def admin_get_services_configuration(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get Registered Services Configuration (AdminGetServicesConfiguration)

    Get Registered Services Configuration.


    Required permission `ADMIN:NAMESPACE:{namespace}:GDPR:CONFIGURATION [READ]` and scope `account`

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:GDPR:CONFIGURATION [READ]

    Required Scope(s):
        - account

    Properties:
        url: /gdpr/admin/namespaces/{namespace}/services/configurations

        method: GET

        tags: ["Configuration"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - DtoServicesConfigurationResponse (OK)

        401: Unauthorized - ResponseError (Unauthorized)

        404: Not Found - ResponseError (Not Found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminGetServicesConfiguration.create(
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminGetServicesConfiguration)
async def admin_get_services_configuration_async(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get Registered Services Configuration (AdminGetServicesConfiguration)

    Get Registered Services Configuration.


    Required permission `ADMIN:NAMESPACE:{namespace}:GDPR:CONFIGURATION [READ]` and scope `account`

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:GDPR:CONFIGURATION [READ]

    Required Scope(s):
        - account

    Properties:
        url: /gdpr/admin/namespaces/{namespace}/services/configurations

        method: GET

        tags: ["Configuration"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - DtoServicesConfigurationResponse (OK)

        401: Unauthorized - ResponseError (Unauthorized)

        404: Not Found - ResponseError (Not Found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminGetServicesConfiguration.create(
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminResetServicesConfiguration)
def admin_reset_services_configuration(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Reset Registered Services Configuration (AdminResetServicesConfiguration)

    [TEST FACILITY ONLY]
    Reset Registered Services Configuration to use the default configuration.


    Required permission `ADMIN:NAMESPACE:{namespace}:GDPR:CONFIGURATION [DELETE]` and scope `account`

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:GDPR:CONFIGURATION [DELETE]

    Required Scope(s):
        - account

    Properties:
        url: /gdpr/admin/namespaces/{namespace}/services/configurations/reset

        method: DELETE

        tags: ["Configuration"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (No Content)

        401: Unauthorized - ResponseError (Unauthorized)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminResetServicesConfiguration.create(
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminResetServicesConfiguration)
async def admin_reset_services_configuration_async(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Reset Registered Services Configuration (AdminResetServicesConfiguration)

    [TEST FACILITY ONLY]
    Reset Registered Services Configuration to use the default configuration.


    Required permission `ADMIN:NAMESPACE:{namespace}:GDPR:CONFIGURATION [DELETE]` and scope `account`

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:GDPR:CONFIGURATION [DELETE]

    Required Scope(s):
        - account

    Properties:
        url: /gdpr/admin/namespaces/{namespace}/services/configurations/reset

        method: DELETE

        tags: ["Configuration"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (No Content)

        401: Unauthorized - ResponseError (Unauthorized)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminResetServicesConfiguration.create(
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminUpdateServicesConfiguration)
def admin_update_services_configuration(
    body: DtoServiceConfigurationUpdateRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update Registered Services Configuration (AdminUpdateServicesConfiguration)

    Update Registered Services Configuration.


    Required permission `ADMIN:NAMESPACE:{namespace}:GDPR:CONFIGURATION [UPDATE]` and scope `account`

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:GDPR:CONFIGURATION [UPDATE]

    Required Scope(s):
        - account

    Properties:
        url: /gdpr/admin/namespaces/{namespace}/services/configurations

        method: PUT

        tags: ["Configuration"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED DtoServiceConfigurationUpdateRequest in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - DtoServiceConfigurationUpdateRequest (OK)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminUpdateServicesConfiguration.create(
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminUpdateServicesConfiguration)
async def admin_update_services_configuration_async(
    body: DtoServiceConfigurationUpdateRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update Registered Services Configuration (AdminUpdateServicesConfiguration)

    Update Registered Services Configuration.


    Required permission `ADMIN:NAMESPACE:{namespace}:GDPR:CONFIGURATION [UPDATE]` and scope `account`

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:GDPR:CONFIGURATION [UPDATE]

    Required Scope(s):
        - account

    Properties:
        url: /gdpr/admin/namespaces/{namespace}/services/configurations

        method: PUT

        tags: ["Configuration"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED DtoServiceConfigurationUpdateRequest in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - DtoServiceConfigurationUpdateRequest (OK)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminUpdateServicesConfiguration.create(
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(DeleteAdminEmailConfiguration)
def delete_admin_email_configuration(
    emails: List[str],
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete admin emails configurations (DeleteAdminEmailConfiguration)

    Delete a list of admin email addresses to stop receiving personal data request notification.


    Required permission `ADMIN:NAMESPACE:{namespace}:EMAIL:CONFIGURATION [DELETE]` and scope `account`

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:EMAIL:CONFIGURATION [DELETE]

    Required Scope(s):
        - account

    Properties:
        url: /gdpr/admin/namespaces/{namespace}/emails/configurations

        method: DELETE

        tags: ["Configuration"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        emails: (emails) REQUIRED List[str] in query

    Responses:
        204: No Content - (No Content)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        404: Not Found - ResponseError (Not Found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteAdminEmailConfiguration.create(
        emails=emails,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(DeleteAdminEmailConfiguration)
async def delete_admin_email_configuration_async(
    emails: List[str],
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete admin emails configurations (DeleteAdminEmailConfiguration)

    Delete a list of admin email addresses to stop receiving personal data request notification.


    Required permission `ADMIN:NAMESPACE:{namespace}:EMAIL:CONFIGURATION [DELETE]` and scope `account`

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:EMAIL:CONFIGURATION [DELETE]

    Required Scope(s):
        - account

    Properties:
        url: /gdpr/admin/namespaces/{namespace}/emails/configurations

        method: DELETE

        tags: ["Configuration"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        emails: (emails) REQUIRED List[str] in query

    Responses:
        204: No Content - (No Content)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        404: Not Found - ResponseError (Not Found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteAdminEmailConfiguration.create(
        emails=emails,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetAdminEmailConfiguration)
def get_admin_email_configuration(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get admin email addresses configuration (GetAdminEmailConfiguration)

    Get list of admin email address configuration.


    Required permission `ADMIN:NAMESPACE:{namespace}:EMAIL:CONFIGURATION [READ]` and scope `account`

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:EMAIL:CONFIGURATION [READ]

    Required Scope(s):
        - account

    Properties:
        url: /gdpr/admin/namespaces/{namespace}/emails/configurations

        method: GET

        tags: ["Configuration"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - List[str] (OK)

        401: Unauthorized - ResponseError (Unauthorized)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetAdminEmailConfiguration.create(
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetAdminEmailConfiguration)
async def get_admin_email_configuration_async(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get admin email addresses configuration (GetAdminEmailConfiguration)

    Get list of admin email address configuration.


    Required permission `ADMIN:NAMESPACE:{namespace}:EMAIL:CONFIGURATION [READ]` and scope `account`

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:EMAIL:CONFIGURATION [READ]

    Required Scope(s):
        - account

    Properties:
        url: /gdpr/admin/namespaces/{namespace}/emails/configurations

        method: GET

        tags: ["Configuration"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - List[str] (OK)

        401: Unauthorized - ResponseError (Unauthorized)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetAdminEmailConfiguration.create(
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(SaveAdminEmailConfiguration)
def save_admin_email_configuration(
    body: List[str],
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Add admin email address configuration (SaveAdminEmailConfiguration)

    Add admin email address for receiving personal data request notification.


    Required permission `ADMIN:NAMESPACE:{namespace}:EMAIL:CONFIGURATION [CREATE]` and scope `account`

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:EMAIL:CONFIGURATION [CREATE]

    Required Scope(s):
        - account

    Properties:
        url: /gdpr/admin/namespaces/{namespace}/emails/configurations

        method: POST

        tags: ["Configuration"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED List[str] in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (No Content)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = SaveAdminEmailConfiguration.create(
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(SaveAdminEmailConfiguration)
async def save_admin_email_configuration_async(
    body: List[str],
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Add admin email address configuration (SaveAdminEmailConfiguration)

    Add admin email address for receiving personal data request notification.


    Required permission `ADMIN:NAMESPACE:{namespace}:EMAIL:CONFIGURATION [CREATE]` and scope `account`

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:EMAIL:CONFIGURATION [CREATE]

    Required Scope(s):
        - account

    Properties:
        url: /gdpr/admin/namespaces/{namespace}/emails/configurations

        method: POST

        tags: ["Configuration"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED List[str] in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (No Content)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = SaveAdminEmailConfiguration.create(
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(UpdateAdminEmailConfiguration)
def update_admin_email_configuration(
    body: List[str],
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update admin email address configuration (UpdateAdminEmailConfiguration)

    Update admin email address for receiving personal data request notification.

    Required permission `ADMIN:NAMESPACE:{namespace}:EMAIL:CONFIGURATION [UPDATE]`

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:EMAIL:CONFIGURATION [UPDATE]

    Properties:
        url: /gdpr/admin/namespaces/{namespace}/emails/configurations

        method: PUT

        tags: ["Configuration"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED List[str] in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (No Content)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UpdateAdminEmailConfiguration.create(
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(UpdateAdminEmailConfiguration)
async def update_admin_email_configuration_async(
    body: List[str],
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update admin email address configuration (UpdateAdminEmailConfiguration)

    Update admin email address for receiving personal data request notification.

    Required permission `ADMIN:NAMESPACE:{namespace}:EMAIL:CONFIGURATION [UPDATE]`

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:EMAIL:CONFIGURATION [UPDATE]

    Properties:
        url: /gdpr/admin/namespaces/{namespace}/emails/configurations

        method: PUT

        tags: ["Configuration"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED List[str] in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (No Content)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UpdateAdminEmailConfiguration.create(
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )
